import os
import os.path
import sys
from setuptools import setup, find_packages
sys.path.insert(0, '.')
from wand.version import VERSION


def readme():
    with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
        return f.read()


wand_includes = [
    "wand",
    "wand.cdefs"
]

wand_excludes = [
    "prof",     # CI Memory profile.
    "reports",  # CI coverage reports.
    "temp",     # CI artifacts.
    "sample",   # Old documents.
    "support",  # Non-public issues.
]

test_requires = [
    'pytest >= 7.2.0',
]

doc_requires = [
    'Sphinx >= 5.3.0',
]

setup(
    name='Wand',
    packages=find_packages(
        include=wand_includes,
        exclude=wand_excludes,
    ),
    version=VERSION,
    description='Ctypes-based simple MagickWand API binding for Python',
    long_description=readme(),
    long_description_content_type='text/x-rst',
    license='MIT',
    license_files = ['LICENSE'],
    author='Hong Minhee',
    author_email='hongminhee' '@' 'member.fsf.org',
    maintainer='E. McConville',
    maintainer_email='emcconville' '@' 'emcconville.com',
    url='http://wand-py.org/',
    extras_require={'doc': doc_requires, 'test': test_requires},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Multimedia :: Graphics'
    ],
    keywords='ImageMagick ctypes',
    project_urls={
        'Documentation': 'https://docs.wand-py.org',
        'Source': 'https://github.com/emcconville/wand',
        'Tracker': 'https://github.com/emcconville/wand/issues',
    }
)



# Why the name of this file does start with underscore?
#
# This tests Wand's internal reference counter, so we can't assume
# the initial state after any function of Wand are used.
# That means this tests have to be first, and pytest automatically
# discovers tests just using filenames.  Fortuneately, it seems to run
# tests in lexicographical order, so we simply adds underscore to
# the beginning of the filename.
from pytest import mark

from wand import exceptions, resource


class DummyResource(resource.Resource):

    def set_exception_type(self, idx):
        self.exception_index = idx

    def get_exception(self):
        exc_cls = exceptions.TYPE_MAP[self.exception_index]
        return exc_cls("Dummy exception")


@mark.parametrize('code', exceptions.TYPE_MAP.keys())
def test_raises_exceptions(recwarn, code):
    """Exceptions raise, and warnings warn"""
    res = DummyResource()
    res.set_exception_type(code)
    try:
        res.raise_exception()
    except exceptions.WandException as e:
        assert not e.__class__.__name__.endswith('Warning')
        assert str(e) == 'Dummy exception'
    else:
        w = recwarn.pop()
        assert w.category.__name__.endswith('Warning')
        assert "Dummy exception" in str(w.message)
        assert recwarn.list == []


def test_limits():
    area_was = resource.limits['area']  # Save state.
    area_expected = area_was - 100
    resource.limits['area'] = area_expected
    assert resource.limits['area'] <= area_expected
    # We have no images loaded, so the current area should be zero.
    assert resource.limits.resource('area') == 0
    del resource.limits['area']
    assert resource.limits['area'] == 0
    resource.limits['area'] = area_was  # To restore for other tests.
    # Non functional smoke test.
    for _ in resource.limits:
        pass
    assert len(resource.limits) > 0
